-----------------------------------------------
-- Simple clock divider
-- by trap15
-----------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

package clock_gen is
	component divider
	generic(n: natural := 16);
		port(	clk:	in std_logic;
			reset:	in std_logic;
			div:	in std_logic_vector(n-1 downto 0);
			Q:	out std_logic);
	end component;

	component clock
		generic(src: natural := 50000000);
		port(	clk:	in std_logic;
			reset:	in std_logic;
			rate:	in natural;
			Q:	out std_logic);
	end component;
end clock_gen;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity divider is
	generic(n: natural := 32);
	port(	clk:	in std_logic;
		reset:	in std_logic;
		div:	in std_logic_vector(n-1 downto 0);
		Q:	out std_logic);
end divider;

architecture behv of divider is
	signal state: std_logic_vector(n downto 0);
	signal res: std_logic;
begin
	process(clk, div, reset)
	begin
		if(reset = '1') then
			state <= (others => '0');
			res <= '0';
		elsif(rising_edge(clk)) then
			if(state = div) then
				res <= not res;
				state <= (others => '0');
			else
				state <= unsigned(state) + 1;
			end if;
		end if;
	end process;
	Q <= res;
end behv;

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_signed.all;

entity clock is
	generic(src: natural := 50000000);
	port(	clk:	in std_logic;
		reset:	in std_logic;
		rate:	in natural;
		Q:	out std_logic);
end clock;

architecture behv of clock is
	signal state: signed(31 downto 0);
	signal res: std_logic;
	signal tmp: std_logic_vector(31 downto 0) := (others => '0');
begin
	process(clk, rate, reset)
	begin
		if(reset = '1') then
			state <= signed(tmp);
			res <= '0';
		elsif(rising_edge(clk)) then
			if(state < 0) then
				state <= state + rate;
			else
				res <= not res;
				state <= state + (rate - src);
			end if;
		end if;
	end process;
	Q <= res;
end behv;
