
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;

ENTITY myand IS
	PORT (	a : in std_logic;
				b : in std_logic;
				z : out std_logic
			);
END myand;

ARCHITECTURE myand1 of myand IS
BEGIN
	z <= a and b;
END;


LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use work.clock_gen.all;
use work.vga.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_signed.all;

ENTITY myfirst IS
	PORT (	
				--Various external IO
				SW : IN STD_LOGIC_VECTOR(9 DOWNTO 0);
				KEY : IN STD_LOGIC_VECTOR(3 DOWNTO 0);
				LEDR : OUT STD_LOGIC_VECTOR(9 DOWNTO 0);
				LEDG : OUT STD_LOGIC_VECTOR(7 DOWNTO 0);
				--EXTERNAL Clocks
				CLOCK_50: IN STD_LOGIC;
				--VGA
				VGA_R: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
				VGA_G: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
				VGA_B: OUT STD_LOGIC_VECTOR(3 DOWNTO 0);
				VGA_HS, VGA_VS : out std_logic;
				--SRAM
				SRAM_ADDR: out std_logic_vector(17 downto 0);
				SRAM_DQ: inout std_logic_vector(15 downto 0);
				SRAM_CE_N, SRAM_LB_N, SRAM_UB_N, SRAM_OE_N, SRAM_WE_N: out std_logic
			);
END myfirst;

ARCHITECTURE impl of myfirst IS
COMPONENT myand
	PORT (	a : in std_logic;
				b : in std_logic;
				z : out std_logic
			);
END COMPONENT;

component clock
	generic(src: natural := 50000000);
	port(	clk:	in std_logic;
		reset:	in std_logic;
		rate:	in natural;
		Q:	out std_logic);
end component;

component divider
generic(n: natural := 16);
	port(	clk:	in std_logic;
		reset:	in std_logic;
		div:	in std_logic_vector(n-1 downto 0);
		Q:	out std_logic);
end component;
	
component vgaSync is
	PORT (	clk25 : in std_logic;	--25.175 (or close enough!) pixel clock
				reset : in std_logic;	--master reset
				hires : in std_logic;	-- 1 if in 640x480 mode, 0 = 320x240
				hSync : out std_logic;	--horizontal sync output
				vSync : out std_logic;	--verticle sync output
				hClock : out std_logic	--horizontal pixel clock (only active when pixels are to be clocked)
			);
end component;

-- Alias list of pin names to internal names
signal globalReset: std_logic;
alias clk50	is CLOCK_50;
alias hires is SW(9);

--signal a1: std_logic;
signal z1: std_logic;
signal hClock: std_logic;
signal clk25: std_logic;
signal rBar, gBar, bBar: std_logic;
signal hSync, vSync: std_logic;
signal ledFlash: std_logic;
signal vramCnt : unsigned (14 downto 0);

BEGIN
	A1: myand PORT MAP (SW(0), SW(1), LEDG(0));
	C1: clock PORT MAP (clk50, globalReset, 5, ledFlash);
	C2: clock PORT MAP (clk50, globalReset, 12, LEDG(6));
	C3: clock PORT MAP (clk25, globalReset, 12, LEDG(5));
	vga: vgaSync PORT MAP (clk25, globalReset, hires, hSync, vSync, hClock); 
	d1: divider PORT MAP (hClock, not hSync, "0000000001010000", bBar);
	d2: divider PORT MAP (hClock, not hSync, "0000000010100000", rBar);
	d3: divider PORT MAP (hClock, not hSync, "0000000101000000", gBar);
	
	LEDG(7) <= ledFlash;
	LEDR(0) <= SW(0);
	LEDR(1) <= SW(1);
	LEDR(2) <= SW(2);
	LEDR(4) <= SW(4) and ledFlash;
	LEDR(5) <= SW(5) and ledFlash;
	LEDR(6) <= SW(6) and ledFlash;
	LEDR(7) <= SW(7) and ledFlash;
	LEDR(9) <= SW(9);
	LEDR(8) <= SW(8);
	
	VGA_HS <= hSync;
	VGA_VS <= vSync;
	
	globalReset <= not KEY(0);	
	
	--gen 25mhz clk
	process (clk50) 
	begin
		if(rising_edge(clk50)) then
			clk25 <= not clk25;
		end if;
	end process;
	
	process (hClock, globalReset) 
	begin
--		if(rising_edge(hClock)) then 
			if(sw(4) = '0') then
				--Show colorbars
				if((rBar = '1') and (SW(7) = '0')) then
					VGA_R <= "1111";
				else 
					VGA_R <= "0000";
				end if;
				if((gBar = '1')  and (SW(6) = '0')) then
					VGA_G <= "1111";
				else 
					VGA_G <= "0000";
				end if;
				if((bBar = '1')  and (SW(5) = '0')) then
					VGA_B <= "1111";
				else 
					VGA_B <= "0000";
				end if;
			else
				if(rising_edge(hClock)) then
					VGA_R <= SRAM_DQ(3 downto 0);
					VGA_G <= SRAM_DQ(7 downto 4);
					VGA_B <= SRAM_DQ(11 downto 8);
				elsif(falling_edge(hClock)) then
					vramCnt <= vramCnt + 1;
					SRAM_ADDR(14 downto 0) <= std_logic_vector(vramCnt);
					SRAM_ADDR(17 downto 15) <= "000";
				end if;
				--Clockout from SRAM (u_u;;
--				VGA_R <= "0000";
--				VGA_G <= "0000";
--				VGA_B <= "0000";
				
			end if;
		if(globalReset = '1') then
			vramCnt <= (others => '0');
		end if;			
--		end if;
	end process;
END;

